﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Pushes a boolean of whether the top vector is empty.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  [StorableClass]
  public abstract class VectorEmptyVectorExpression<T> : StatelessExpression {
    protected VectorEmptyVectorExpression() { }
    [StorableConstructor]
    protected VectorEmptyVectorExpression(bool deserializing) : base(deserializing) { }

    protected bool IsNoop(IPushStack<List<T>> vectorStack) {
      return vectorStack.IsEmpty;
    }

    protected void Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack) {
      var vector = vectorStack.Pop();
      interpreter.BooleanStack.Push(vector.Count == 0);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.IntegerVector, "INTEGER[].EMPTYVECTOR", StackTypes.Boolean)]
  public class IntegerVectorEmptyVectorExpression : VectorEmptyVectorExpression<long> {
    public IntegerVectorEmptyVectorExpression() { }
    [StorableConstructor]
    protected IntegerVectorEmptyVectorExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter.IntegerVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.IntegerVectorStack);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.FloatVector, "FLOAT[].EMPTYVECTOR", StackTypes.Boolean)]
  public class FloatVectorEmptyVectorExpression : VectorEmptyVectorExpression<double> {
    public FloatVectorEmptyVectorExpression() { }
    [StorableConstructor]
    protected FloatVectorEmptyVectorExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter.FloatVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.FloatVectorStack);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].EMPTYVECTOR", StackTypes.Boolean)]
  public class BooleanVectorEmptyVectorExpression : VectorEmptyVectorExpression<bool> {
    public BooleanVectorEmptyVectorExpression() { }
    [StorableConstructor]
    protected BooleanVectorEmptyVectorExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter.BooleanVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.BooleanVectorStack);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.StringVector, "STRING[].EMPTYVECTOR", StackTypes.Boolean)]
  public class StringVectorEmptyVectorExpression : VectorEmptyVectorExpression<string> {
    public StringVectorEmptyVectorExpression() { }
    [StorableConstructor]
    protected StringVectorEmptyVectorExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter.StringVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.StringVectorStack);
    }
  }
}
