﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Problem {
  using Common;
  using Configuration;
  using Core;
  using Encodings.IntegerVectorEncoding;
  using Expressions;
  using Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class PushSolution : NamedItem {
    [Storable]
    public readonly double Quality;
    [Storable]
    public readonly IntegerVector IntegerVector;
    [Storable]
    public readonly IRandom Random;
    [Storable]
    public readonly IReadOnlyPushConfiguration Config;
    [Storable]
    public readonly IPushEvaluator Evaluator;
    [Storable]
    protected readonly bool simplify;

    public PushProgram Program;


    public PushSolution(IntegerVector integerVector, double quality, IRandom random, IReadOnlyPushConfiguration config, IPushEvaluator evaluator, bool simplify = false)
      : base("Solution", "A push solution.") {
      IntegerVector = integerVector;
      Quality = quality;
      Random = random;
      Config = config;
      Evaluator = evaluator;
      this.simplify = simplify;

      InitProgram();
    }

    private void InitProgram() {
      Program = IntegerVector.ToPushProgram(Config);

      if (simplify)
        Program = Simplifier.Simplifier.Simplify(Program, Config, p => Evaluator.EvaluateTraining(Config, p, Random).TotalQuality);
    }

    public PushSolution(PushSolution origin, Cloner cloner) : base(origin, cloner) {
      IntegerVector = cloner.Clone(origin.IntegerVector);
      Quality = origin.Quality;
      Random = cloner.Clone(origin.Random);
      Config = cloner.Clone(origin.Config);
      Evaluator = cloner.Clone(origin.Evaluator);
      Program = origin.Program; // push programs are immutable like all expressions
      simplify = origin.simplify;
    }

    public virtual PushSolution Simplify() {
      return new PushSolution(IntegerVector, Quality, Random, Config, Evaluator, true);
    }

    [StorableConstructor]
    protected PushSolution(bool deserializing) : base(deserializing) { }

    [StorableHook(HookType.AfterDeserialization)]
    // ReSharper disable once UnusedMember.Local
    private void AfterDeserialization() {
      InitProgram();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PushSolution(this, cloner);
    }
  }
}
